package org.khelekore.jsizer;

import java.util.ArrayList;
import java.util.List;

/** A graph builder that uses full class names to build the graph.
 */
public class FullNameGraphBuilder implements GraphBuilder {
    public boolean hasFullName () {
	return false;
    }

    public List<TreeNode> buildGraph (List<ClassSize> classSizes, 
				      TreeNodeFactory factory) {
	List<TreeNode> treeNodes = new ArrayList<TreeNode> ();
	for (ClassSize cs : classSizes)
	    addNode (cs, factory, treeNodes);
	return treeNodes;
    }

    private void addNode (ClassSize cs, TreeNodeFactory factory, 
			  List<TreeNode> treeNodes) {
	String name = cs.getClassName ();
	String[] names = name.split ("[.$]");
	TreeNode parent = null;
	for (int i = 0; i < names.length - 1; i++) {
	    TreeNode tn = findNode (factory, parent, names[i], treeNodes);
	    parent = tn;
	}
	String shortName = names[names.length - 1];
	TreeNode tn = factory.create (cs, shortName);
	addNode (parent, tn, treeNodes);
    }

    private TreeNode findNode (TreeNodeFactory factory, TreeNode parent, 
			       String name, List<TreeNode> treeNodes) {
	List<TreeNode> ls = 
	    parent == null ? treeNodes : parent.getChildren ();
	for (TreeNode tn : ls)
	    if (tn.getName ().equals (name))
		return tn;
	TreeNode ntn = factory.create (name);
	addNode (parent, ntn, treeNodes);
	return ntn;
    }

    private void addNode (TreeNode parent, TreeNode tn, List<TreeNode> treeNodes) {
	if (parent == null)
	    treeNodes.add (tn);
	else 
	    parent.addChild (tn);
    }
}
