package org.khelekore.jsizer;

import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

/** A popup menu handler.  */
class PopupHandler extends MouseAdapter {
    private SizePanel sp;
    
    public PopupHandler (SizePanel sp) {
	this.sp = sp;
    }

    @Override public void mousePressed (MouseEvent e) {
	if (!SwingUtilities.isRightMouseButton(e)) 
	    return;
	JPopupMenu p = new JPopupMenu ();
	p.add (new JMenuItem (new SetNullFilter ()));
	
	List<TreeNode> tns = sp.getNodesAt (e);
	if (!tns.isEmpty ()) {
	    GraphBuilder gb = sp.getGraphBuilder ();
	    if (gb.hasFullName ()) {
		addFullNameItems (p, tns);
	    } else {
		addHierarchyItems (p, tns);
	    }
	}

	p.show (sp, e.getX (), e.getY ());
    }

    private void addFilterItems (JPopupMenu p, List<String> names) {
	StringBuilder sb = new StringBuilder ();
	for (String partialName : names) {
	    if (sb.length () > 0)
		sb.append ('.');
	    sb.append (partialName);
	    p.add (new JMenuItem (new SetPrefixAction (sb.toString ())));	    
	}
    }
    
    private void addHierarchyItems (JPopupMenu p, List<TreeNode> tns) {
	List<String> names = new ArrayList<String> (tns.size ());
	for (TreeNode tn : tns)
	    names.add (tn.getName ());
	addFilterItems (p, names);
    }

    private void addFullNameItems (JPopupMenu p, List<TreeNode> tns) {
	TreeNode tn = tns.get (tns.size () - 1);
	String name = tn.getName ();
	String[] parts = name.split ("[.$]");
	addFilterItems (p, Arrays.asList (parts));
    }

    private class SetNullFilter extends AbstractAction {
	private static final long serialVersionUID = 1;	

	public SetNullFilter () {
	    super ("Clear name filter");
	}

	public void actionPerformed (ActionEvent e) {
	    sp.setClassNameFilter (null);
	}
    }

    private class SetPrefixAction extends AbstractAction {
	private static final long serialVersionUID = 1;	
	private String prefix;

	public SetPrefixAction (String prefix) {
	    super ("Set class name filter to: " + prefix);
	    this.prefix = prefix;
	}

	public void actionPerformed (ActionEvent e) {
	    sp.setClassNameFilter (new PrefixFilter (prefix));
	}
    }
}
