package org.khelekore.jsizer;

/** A filter that removes too small classes. */
public class SizeFilter {
    private final int minNumFields;
    private final int minNumMethods;
    private final int minNumMethodsLine;
    private final int minNumUsedClasses;
    
    public SizeFilter (int minNumFields, int minNumMethods, 
		       int minNumMethodsLine, int minNumUsedClasses) {
	this.minNumFields = minNumFields;
	this.minNumMethods = minNumMethods;
	this.minNumMethodsLine = minNumMethodsLine;
	this.minNumUsedClasses = minNumUsedClasses;
    }

    public boolean accept (ClassSize cs) {
	return 
	    (minNumFields > 0 && cs.getNumFields () >= minNumFields) ||
	    (minNumMethods > 0 && cs.getNumMethods () >= minNumMethods) ||
	    (minNumMethodsLine > 0 && 
	     cs.getNumMethodLines () >= minNumMethodsLine) || 
	    (minNumUsedClasses > 0 && 
	     cs.getNumUsedClasses () >= minNumUsedClasses);
    }

    public int getMinFields () {
	return minNumFields;
    }

    public int getMinMethods () {
	return minNumMethods;
    }

    public int getMinLines () {
	return minNumMethodsLine;
    }

    public int getMinUsedClasses () {
	return minNumUsedClasses;
    }
}
