package org.khelekore.jsizer;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/** Holder of total size information  */
public class SizeHandler implements Serializable {
    private int maxFields;
    private int maxMethods;
    private int maxUsedClasses;
    private int maxLines;
    private int totalWidth;
    private int totalHeight;

    public int getMaxFields () {
	return maxFields;
    }

    public int getMaxMethods () {
	return maxMethods;
    }

    public int getMaxUsedClasses () {
	return maxUsedClasses;
    }

    public int getMaxLines () {
	return maxLines;
    }

    public int getTotalWidth () {
	return totalWidth;
    }

    public int getTotalHeight () {
	return totalHeight;
    }

    public List<ClassSize> findFilteredClassSizes (List<ClassSize> classSizes,
						   SizeFilter sizeFilter,
						   ClassNameFilter classNameFilter) {
	List<ClassSize> clss = classSizes;
	if (sizeFilter != null || classNameFilter != null)
	    clss = findFilteredClasses (classSizes, sizeFilter, classNameFilter);
	else
	    setMaxValues (classSizes);
	return clss;
    }

    public void clearSize () {
	totalWidth = 0;
	totalHeight = 0;
    }

    public void addToTotalWidth (int width) {
	totalWidth += width;
    }

    public void maximizeHeight (int height) {
	totalHeight = Math.max (totalHeight, height);
    }

    private void clearMaxValues () {
	maxFields = 0;
	maxMethods = 0;
	maxLines = 0;
	maxUsedClasses = 0;
    }

    private void updateMaxValues (ClassSize cs) {
	maxFields = Math.max (maxFields, cs.getNumFields ());
	maxMethods = Math.max (maxMethods, cs.getNumMethods ());
	maxLines = Math.max (maxLines, cs.getNumMethodLines ());
	maxUsedClasses = Math.max (maxUsedClasses, cs.getNumUsedClasses ());
    }

    private void setMaxValues (List<ClassSize> classSizes) {
	clearMaxValues ();
	for (ClassSize cs : classSizes)
	    updateMaxValues (cs);
    }

    /** Find classes filtered out, but also add the parent
     *  classes that we can find.
     */
    private List<ClassSize> findFilteredClasses (List<ClassSize> classSizes,
						 SizeFilter sizeFilter,
						 ClassNameFilter classNameFilter) {
	clearMaxValues ();
	List<ClassSize> ls = new ArrayList<ClassSize> (classSizes.size ());
	Map<String, ClassSize> skippedClasses =
	    new HashMap<String, ClassSize> (classSizes.size ());
	for (ClassSize cs : classSizes) {
	    if (classNameFilter != null &&
		!classNameFilter.accept (cs.getClassName ()))
		continue;
	    if (sizeFilter == null || sizeFilter.accept (cs)) {
		updateMaxValues (cs);
		ls.add (cs);
		String cn = cs.getClassName ();
		int i = indexOfLastSplitter (cn);
		int j = 1;
		while (i > 0) {   // need at least one char before '.'
		    cn = cn.substring (0, i);
		    ClassSize pcs = skippedClasses.get (cn);
		    if (pcs == null) // already in...
			break;
		    ls.add (ls.size () - j, pcs);
		    i = indexOfLastSplitter (cn);
		}
	    } else {
		skippedClasses.put (cs.getClassName (), cs);
	    }
	}
	return ls;
    }

    // return the index of the last class name splitter,
    // return -1 if none found.
    private int indexOfLastSplitter (String s) {
	return Math.max (s.lastIndexOf ('.'), s.lastIndexOf ('$'));
    }
}
